#!/usr/bin/perl
require 5.005;
use strict;

use File::Basename;
use lib dirname($0);
use Tables;
use DBI;

#
# some global variables, adjust according to your needs:
#
# $dbhost: the FQDN of the host where the database server resides.
# $dbname: the name of the database driver you are using.
# $dbuser: the username of the login account to the database
# $dbpass: the password for the login onto the database
# $dborg: The organization as needed for DBase authentication
my ($dbhost,$dbname,$dbuser,$dbpass,$dborg);
$dbhost = 'hawaii.rvs.uni-hannover.de';
$dbname = 'Oracle';
$dbuser = '';
$dbpass = '';
($dbuser,$dbpass) = split(/\//,$ENV{ORACLE_USERID});
$dborg = $ENV{ORACLE_SID} || $ENV{TWO_TASK} || 'RVS';

# associate the high level abstract interface with the API driver
print STDERR "# installing database driver\n";
my $driver = DBI->install_driver($dbname) || die "not ok: $DBI::errstr\n";

# connect to your database
print STDERR "# trying to connect to DB\n";
$main::handle = $driver->connect( $dborg, $dbuser, $dbpass,
			    { RaiseError => 1, AutoCommit => 0 } ) || 
    die "can\'t connect: $DBI::errstr\n";

#
# some error handling in order to roll back mistaken insertions
#
BEGIN {
    $main::skip = -1;
}

$main::skip = 0;
$SIG{INT} = sub { exit(1); };

END {
    $main::handle->rollback() if $main::skip == 0;
}

#
# --------------------------------------------------------------- subs
#

sub check_tables ($) {
    # purpose: check for presence of necessary tables in the database
    # paramtr: $handle (IN): query handle for the database
    # returns: dies on missing tables, or returns 1 for success.
    my $handle = shift;
    my $qh = $handle->prepare( "select table_name from sys.user_tables" ) ||
	die "request failure: $DBI::errstr\n";
    my $rc = $qh->execute() || die "not ok: $DBI::errstr\n";

    my (%tables,@row);
    while ( @row = $qh->fetchrow_array ) {
	print STDERR "# found [", join( ':', @row ), "]\n";
	$tables{uc($row[0])}++;
    }

    my $table;
    foreach $table ( @main::tables ) {
	if ( $tables{$table} != 1 ) {
	    die "Table $table is missing in the database. It seems as if\n" . \
		"the DB creation script was not run. Please create the\n" . \
		"necessary tables and rerun $0.\n";
	}
    }
    1;
}

sub new_timestamp ($$$$) {
    # purpose: check for existing entry and generate a new (unique) tsid
    # paramtr: $nr (IN): stamp number
    #          $handle (IN): database handle
    #          $chost (IN): name of (virtual) cache (group)
    #          $stamp (IN): time stamp as reported by seafood
    my ($nr,$handle,$chost,$stamp) = @_;
    my @row;

    # check, if the data was already submitted into the database
    my $ch = $handle->prepare( "select I${nr}ID from sf_stamp$nr where " .
			       "cache = \'$chost\' and first = $stamp " ) ||
	die "newstamp->prepare: $DBI::errstr\n";
    $ch->execute() ||
	die "newstamp->execute: $DBI::errstr\n";
    @row = $ch->fetchrow_array;
    $ch->finish() ||
	die "newstamp->finish: $DBI::errstr\n";
    if ( defined $row[0] ) {
	warn "WARNING: key($chost,$stamp) as stamp$nr already in database!\n";
	die "WARNING: replace me at your own risk!";
    }

    # update time stamp
    my $uh = $handle->prepare( "update sf_meta set tsid$nr = tsid$nr + 1" ) || 
	die "newstamp->prepare: $DBI::errstr\n";
    $uh->execute() || 
	die "newstamp->update: $DBI::errstr\n";
    $uh->finish;

    # select last time stamps
    my $qh = $handle->prepare( "select tsid$nr from sf_meta" ) ||
	die "tsid->prepare($DBI::errstr)\n";
    $qh->execute() || 
	die "tsid->execute($DBI::errstr)\n";
    @row = $qh->fetchrow_array;
    $qh->finish;

    # print STDERR "# new stamp$nr ID is $row[0]\n";
    $row[0];
}

sub insert_stamp1 ($$@) {
    # purpose: insert first line, and return the $tsid for this file...
    # paramtr: $handle (IO): database connection handle
    #          @args (IN): arguments to the stamp1 input
    my ($handle,$tsid,@args) = @_;
    my $cache = shift @args;

    # insert new values into db
    Tables::insert( $handle, 'sf_stamp1', $tsid, "\'$cache\'", @args );
}

sub insert_stamp2 ($$@) {
    # purpose: insert first line, and return the $tsid for this file...
    # paramtr: $handle (IO): database connection handle
    #          @args (IN): arguments to the stamp2 input
    my ($handle,$tsid,@args) = @_;
    my $cache = shift @args;

    # insert new values into db
    Tables::insert( $handle, 'sf_stamp2', $tsid, "\'$cache\'", @args );
}

#
# purpose: jumptable destinations, inserting new values into a table
# paramtr: $handle (IN): generic database handle/connection
#          $stamp (IN): time stamp (either stamp1 or stamp2)
#          @_ (IN): further arguments, usually contains another key
# returns: result of insertion attempt
#

sub insert_tcp ($$@) {
    my ($handle,$tsid,@args) = @_;
    my $what = shift @args;
    my $stamp = shift @args; # remove timestamp
    my $status = shift @args;

    # insert new values into db
    Tables::insert( $handle, "sf_tcp_$what", $tsid, "\'$status\'", @args );
}

sub insert_udp ($$@) {
    my ($handle,$tsid,@args) = @_;
    my $what = shift @args;
    my $stamp = shift @args; # remove timestamp
    my $status = shift @args;

    # insert new values into db
    Tables::insert( $handle, "sf_udp_$what", $tsid, "\'$status\'", @args );
}

sub insert_hier ($$@) {
    my ($handle,$tsid,@args) = @_;
    my $what = shift @args;
    my $stamp = shift @args; # remove timestamp
    my $peercode = shift @args;

    # insert new values into db
    $args[0] = "\'$args[0]\'" if ( $what eq 'peer' || $what eq 'parent' );
    Tables::insert( $handle, "sf_hier_$what", $tsid, "\'$peercode\'", @args );
}

sub insert_method ($$@) {
    my ($handle,$tsid,@args) = @_;
    my $stamp = shift @args; # remove timestamp
    my $method = shift @args;

    # insert new values into db
    Tables::insert( $handle, 'sf_method', $tsid, "\'$method\'", @args );
}

sub insert_scheme ($$@) {
    my ($handle,$tsid,@args) = @_;
    my $stamp = shift @args; # remove timestamp
    my $scheme = shift @args;

    # insert new values into db
    Tables::insert( $handle, 'sf_scheme', $tsid, "\'$scheme\'", @args );
}

sub insert_tld ($$@) {
    my ($handle,$tsid,@args) = @_;
    my $stamp = shift @args; # remove timestamp
    my $tld = shift @args;

    # insert new values into db
    Tables::insert( $handle, 'sf_tld', $tsid, "\'$tld\'", @args );
}

sub insert_sld ($$@) {
    my ($handle,$tsid,@args) = @_;
    my $stamp = shift @args; # remove timestamp
    my $sld = shift @args;

    # insert new values into db
    Tables::insert( $handle, 'sf_sld', $tsid, "\'$sld\'", @args );
}

sub insert_mime ($$@) {
    my ($handle,$tsid,@args) = @_;
    my $stamp = shift @args; # remove timestamp
    my $mimetype = shift @args;

    # insert new values into db
    Tables::insert( $handle, 'sf_mime', $tsid, "\'$mimetype\'", @args );
}

sub insert_client ($$@) {
    my ($handle,$tsid,@args) = @_;
    my $what = shift @args;	# 'tcp' || 'udp'
    my $stamp = shift @args; # remove timestamp
    my $host = shift @args;

    # insert new values into db
    Tables::insert( $handle, "sf_${what}_client", $tsid, "\'$host\'", @args );
}

sub insert_as ($$@) {
    my ($handle,$tsid,@args) = @_;
    my $asn = shift @args;
    my $stamp = shift @args; # remove timestamp

    # find stop of description string, which is enclosed in quotes
    my ($i,$re) = (0, chr(0x22) . chr(0x24) ); 
    ++$i while ( $i < @args && $args[$i] !~ /$re/ ); 
    my $descr = $i==0 ? $args[0] : join(' ',@args[0,$i]);
    $descr =~ s/^"/'/; # '"
    $descr =~ s/"$/'/; # '"

    # insert new values into db
    $i = @args-1;
    Tables::insert( $handle,'sf_as',$tsid,"\'$asn\'",$descr,@args[$i-2..$i] );
}

sub insert_peak ($$@) {
    my ($handle,$tsid,@args) = @_;
    my $stamp = shift @args; # remove timestamp

    # insert new values into db
    Tables::insert( $handle, 'sf_peak', $tsid, @args );
}

#
# --------------------------------------------------------------- main
#
    
my %jumptable = 
    ( 
      'tcp' => \&insert_tcp,
      'udp' => \&insert_udp,
      'hier' => \&insert_hier,
      'method' => \&insert_method,
      'scheme' => \&insert_scheme,
      'tld' => \&insert_tld,
      'sld' => \&insert_sld,
      'mime' => \&insert_mime,
      'client' => \&insert_client,
      'as' => \&insert_as
    );

print STDERR "# trying to verify tables.\n";
check_tables($main::handle);

print STDERR "# reading input\n";
my (@x,$key,$lastkey,$tsid1,$tsid2);
while ( <> ) {
    chomp;
    @x = split/ +/;
    $key = lc(shift @x);
    die "illegal key, check the input" unless defined $key;

    # just a short debug output, report each key once, if possible.
    if ( $lastkey ne $key ) {
	print STDERR "# $key...\n";
	$lastkey = $key;
    }

    # the time stamps and peaks need special handling, 
    # the rest will be done via a jumptable approach.
    if ( $key eq 'stamp1' ) {
	$tsid1 = new_timestamp( 1, $main::handle, $x[0], $x[1] );
	insert_stamp1( $main::handle, $tsid1, @x );
    } elsif ( $key eq 'stamp2' ) {
	$tsid2 = new_timestamp( 2, $main::handle, $x[0], $x[1] );
	insert_stamp2( $main::handle, $tsid2, @x );
    } elsif ( $key eq 'peak' ) {
	insert_peak( $main::handle, $tsid2, @x );
    } else {
	my $callref = $jumptable{$key};
	&{$callref}( $main::handle, $tsid1, @x ) if ( defined $callref );
    }
}

$main::handle->commit() ||
    die "commit: $DBI::errstr\n";
$main::handle->disconnect() ||
    die "disconnect: $DBI::errstr\n";
$main::skip = 1;
print "SUCCESS!\n";
